#!/usr/bin/env python3
import logging
import yaml
from pequod_agent.cluster.controllers.base import Controller
from pequod_agent.docker_manager import DockerManager
from pequod_agent.models.application import ApplicationVersion, parse_application_spec
from pequod_agent.models.cluster_state import ClusterState
from pequod_agent.utils import generate_random_name


class BootstrapController(Controller):
    """
    Controller to run in standalone mode to bootstrap management nodes
    """

    def __init__(self, host: str, bootstrap_identifiers: list):
        self.log = logging.getLogger('pequod.controller')
        self.hostname = host
        self.instances = []
        for bootstrap_identifier in bootstrap_identifiers:
            try:
                app = parse_application_spec(bootstrap_identifier)
            except ValueError:
                app = None
            if app:
                instance = self._load_docker_image(app)
                self.instances.append(instance)
            else:
                instances = self._load_bootstrap_file(bootstrap_identifier)
                self.instances += instances

    def _load_bootstrap_file(self, bootstrap_file: str) -> list:
        with open(bootstrap_file) as fd:
            data = yaml.safe_load(fd)
            instances = data.get('instances', [])
            if not isinstance(instances, list):
                raise ValueError('Invalid instances in bootstrap file')
            for inst in instances:
                if not isinstance(inst['instance_name'], str):
                    raise ValueError('Instance name must be a string')
                inst['lifecycle_phase'] = 'START'
                inst['node_name'] = self.hostname
                if 'zone_name' not in inst:
                    inst['zone_name'] = 'bootstrap'
        return instances

    def _load_docker_image(self, app: ApplicationVersion) -> dict:
        if app.repository_name:
            image_template = '{a.repository_name}/{a.application_name}:{a.application_version}'
        else:
            image_template = '{a.application_name}:{a.application_version}'
        image = image_template.format(a=app)
        docker = DockerManager()
        manifest = docker.get_manifest_from_image(image)
        resources = {}
        for res, val in manifest.resources.items():
            for k, v in val.items():
                if k.startswith('default_'):
                    resources['{}_{}'.format(res, k.split('_')[1])] = v
        name_prefix = 'bootstrap-{}-{}-{}-'.format(app.repository_name,
                                                   app.application_name,
                                                   app.application_version)[:24]
        instance_name = generate_random_name(name_prefix, 4)
        instance = {'instance_name': instance_name,
                    'repository_name': app.repository_name,
                    'application_name': app.application_name,
                    'application_version': app.application_version,
                    'node_name': self.hostname,
                    'lifecycle_phase': 'START',
                    'zone_name': 'bootstrap',
                    'resources': resources}
        return instance

    def register(self, node_host: str, node_state: dict):
        self.log.debug('Register: %s', node_state)
        for running_inst in node_state['instances']:
            for inst in self.instances:
                if running_inst['instance_name'] == inst['instance_name']:
                    inst['ipv6'] = running_inst['metadata']['ipv6']

    def update_node(self, node_state):
        self.log.debug('Update: %s', node_state.public_dict)

    def update_instance(self, instance):
        self.log.debug('Update: %s', instance.public_dict)
        for inst in self.instances:
            if instance.name == inst['instance_name']:
                inst['ipv6'] = instance.ip_address

    def delete_instance(self, instance_name):
        self.log.debug('Delete: %s', instance_name)

    def poll(self):
        self.log.debug('Poll: %s', self.instances)
        return ClusterState({self.hostname: {'instances': self.instances}})
