import logging
from pequod_agent.cluster.service import ClusterService
from pequod_agent.constants.ports import CONTROLLER_PORT, MANAGER_PORT, REGISTRY_PORT, CASSANDRA_PORT
from pequod_agent.networking.routing.service import ServiceEndpoint
import requests

CORE_PORTS = {
    'core-manager': MANAGER_PORT,
    'application-registry': REGISTRY_PORT,
    'cluster-controller': CONTROLLER_PORT,
    'core-cassandra': CASSANDRA_PORT,
}


def parse_header_ip(s: str) -> str:
    return s.strip().rsplit(':', 1)[0].strip('[]')


class CoreService(ClusterService):
    def __init__(self, core_ips, service_name, credentials):
        super().__init__(credentials)
        self.log = logging.getLogger('pequod.core-service')
        self.core_ips = set(core_ips)
        self.master_ip = None
        self.service_name = service_name
        self.port = CORE_PORTS[service_name]
        self.on_master_change = None

    @property
    def endpoints(self):
        if self.master_ip:
            return [ServiceEndpoint(self.master_ip, self.port)]
        else:
            return [ServiceEndpoint(ip, self.port) for ip in self.core_ips]

    @property
    def name(self):
        return self.service_name

    def master_update_hook(self):
        """
        Action to execute when changing master
        """
        if self.on_master_change:
            self.on_master_change()

    def _request(self, method_name, method, path, use_credentials=True, *args, **kwargs):
        try:
            response = super()._request(method_name, method, path, use_credentials, *args, **kwargs)
            if self.last_response_headers:
                members = self.last_response_headers.get('X-Pequod-Members')
                if members:
                    for member in members.split(','):
                        self.core_ips.add(parse_header_ip(member))
            return response
        except requests.ConnectionError as e:
            if self.master_ip:
                self.master_ip = None
                return self._request(method_name, method, path, use_credentials, *args, **kwargs)
            raise
        except requests.HTTPError as e:
            if e.response.status_code == 451:
                # we are contacting a service which is not the master
                master = e.response.headers.get('X-Pequod-Master')
                if not master:
                    self.log.warning('Service %s (%s) returned 451 but X-Pequod-Master not found',
                                     self.name, e.response.url)
                    raise
                self.log.info('Service %s (%s) returned 451, switching to new master %s..',
                              self.name, e.response.url, master)
                self.master_ip = parse_header_ip(master)
                self.master_update_hook()
                return super()._request(method_name, method, path, use_credentials, *args, **kwargs)
            raise