#!/usr/bin/env python3

import docker
import io
import logging
import os
import tarfile
import time
from pequod_agent.models.manifest import Manifest, read_manifest_xml
from pequod_agent.utilities.metaclasses import Singleton

MANIFEST_FILE = '/pequod.xml'


class DockerManager(metaclass=Singleton):
    """
    Singleton wrapper around Docker.client
    """
    def __init__(self):
        self.log = logging.getLogger('pequod.docker-manager')
        self.client = docker.Client(timeout=30)
        self.credentials = None

    def __getattr__(self, item):
        """
        If DockerManager doesn't have the method return client method
        """
        return getattr(self.client, item)

    def login(self, registry):
        if not registry.startswith('http://'):
            registry = 'http://' + registry
        if not registry.endswith('/v1/'):
            registry += '/v1/'
        self.log.info('Trying to login at Docker registry %s..', registry)
        self.client.login(self.credentials['username'], self.credentials['password'], registry=registry)

    def pull(self, image: str, tag: str):
        if self.credentials:
            parts = image.split('/')
            self.login(parts[0])
        # TODO: "insecure_registry" should be removed as soon as we have SSL everywhere
        return self.client.pull(image, tag, stream=True, insecure_registry=True)

    def set_timeout(self, timeout: int):
        self.client._timeout = timeout

    @property
    def all_containers(self) -> list:
        return self.client.containers(all=True)

    @property
    def containers(self) -> list:
        return self.client.containers()

    def container_is_running(self, container):
        if isinstance(container, dict):
            container = container.get('Id')
        container_ids = [c['Id'] for c in self.containers]
        return container in container_ids

    def find_container_by_name(self, name: str):
        for container in self.all_containers:
            if container['Names'][0] == '/' + name:
                return container

    def get_manifest_from_image(self, docker_image_name: str) -> Manifest:
        container = self.client.create_container(docker_image_name)
        res = self.client.copy(container, MANIFEST_FILE)
        fd = io.BytesIO(res.read())
        tar = tarfile.open(fileobj=fd)
        buf = tar.extractfile(os.path.basename(MANIFEST_FILE))
        xml = buf.read().decode('utf-8')
        manifest = read_manifest_xml(xml)
        return manifest

    def inspect_container(self, container) -> dict:
        return self.client.inspect_container(container)

    def remove_container_by_name(self, name: str, max_tries: int=10):
        for i in range(max_tries):
            container = self.find_container_by_name(name)
            if container:
                # noinspection PyBroadException
                try:
                    self.log.debug('Trying to remove %s..', container['Id'])
                    self.client.remove_container(container['Id'])
                    return
                except:
                    self.log.info('Failed to remove %s, trying again (%d/%d)..', container['Id'][:12], i, max_tries)
                    time.sleep(1)
            else:
                raise ValueError('Container not found')