#!/usr/bin/env python3
import json
import random
import time
from asyncio import coroutine, Future, sleep
from logging import getLogger
from pequod_agent.docker_manager import DockerManager
from pequod_agent.utilities.metaclasses import Singleton


class ImageDownloader(metaclass=Singleton):

    def __init__(self):
        self.log = getLogger('pequod.image-downloader')
        self.downloading = dict()
        self.docker = DockerManager()

    @coroutine
    def pull_image(self, application_name: str, image_names: list, tag: str):
        self.log.info('Pulling Docker image %s:%s..', application_name, tag)
        key = application_name, tag
        if key in self.downloading:
            self.log.debug('Image %s:%s already being pulled. Waiting..', application_name, tag)
            future = self.downloading[key]
            yield from sleep(0.1)
        else:
            future = Future()
            self.downloading[key] = future

            random.shuffle(image_names)
            last_exception = None
            for image in image_names:
                self.log.debug('Trying to pull %s:%s', image, tag)
                last_yield = 0
                try:
                    yield from sleep(0.1)
                    had_error = False
                    for line in self.docker.pull(image, tag):
                        now = time.time()
                        if now > last_yield + 10:
                            self.log.debug('Yielding')
                            last_yield = now
                            yield from sleep(0.1)
                        data = json.loads(line)
                        if 'error' in data:
                            msg = 'Error while pulling Docker image: {}'.format(data['error'])
                            self.log.warning(msg)
                            had_error = True
                            last_exception = Exception(msg)
                        elif data['status'].lower() == 'pulling fs layer':
                            self.log.info('Pulling FS layer %s..', data['id'])
                        else:
                            self.log.debug('%s', data)
                    if not had_error:
                        future.set_result(image)
                        del self.downloading[key]
                        break
                except Exception as e:
                    self.log.debug('Failed to pull %s:%s', image, tag)
                    last_exception = e
            else:
                self.log.warning('Failed to pull image from any registry: %s', last_exception)
                future.set_result(image_names[0])
                del self.downloading[key]
        yield from future
        image = future.result()
        self.log.info('Finished pulling Docker image %s:%s..', image, tag)
        return image