#!/usr/bin/env python3
import logging

import netaddr
import netifaces
from itertools import chain
from pequod_agent.utilities.system import execute
from yaml import safe_load


class NetworkCard:
    def __init__(self, device: str):
        self.log = logging.getLogger('pequod.network-card')
        self.device = device
        self.settings = self._get_settings()

    def _get_settings(self):
        # if we replace tabs with spaces ethtool output is valid yaml, hacky but it seems to work
        raw_info = execute('ethtool {dev}', _show_output=False, dev=self.device).replace('\t', '    ')
        try:
            info = safe_load(raw_info)
            settings = info['Settings for {dev}'.format(dev=self.device)]
        except Exception as e:
            self.log.warning('Could not read ethtool output: %s, raw output was: "%s"', e, raw_info)
            settings = None
        return settings

    @property
    def speeds_mbits(self) -> int:
        """
        Returns the speed in MBit/s
        """
        try:
            speed = int(self.settings['Speed'].replace('Mb/s', ''))
        except Exception as e:
            self.log.warning('Could not detect network card speed, assuming 100Mb/s: %s', e)
            speed = 100
        return speed

    @classmethod
    def get_first_network_card(cls):
        for device in netifaces.interfaces():
            addresses = list(chain.from_iterable(addresses
                             for family, addresses
                             in netifaces.ifaddresses(device).items()
                             if family in [netifaces.AF_INET, netifaces.AF_INET6]))
            ips = (netaddr.IPAddress(address['addr']) for address in addresses if '%' not in address['addr'])
            is_loopback = any(ip.is_loopback() for ip in ips)
            if not is_loopback and not device.startswith('pequod'):
                return cls(device)